/**
 * Define api of apeman-api-coz
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const coz = require('coz')
const aslogger = require('aslogger')

const spec = require('./spec')

/** @lends define */
function define (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Render bud files
     * @method render
     * @param {string} pattern - Glob filename pattern
     * @param {Object} [options={}] - Optional settings
     * @returns {*}
     */
    render (pattern, options = {}) {
      const s = this
      return s.renderAll([ pattern ], options)
    },

    /**
     * Render multiple files at once
     * @method renderAll
     * @param {string} patterns - Glob filename pattern
     * @param {Object} [options={}] - Optional settings
     * @returns {*}
     */
    renderAll (patterns, options = {}) {
      const s = this
      return co(function * () {
        return yield coz.render(patterns, options)
      })
    },

    /**
     * Interceptor before method invoking
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * Interceptor after method invoking
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
